
#include <Windows.h>
#include "Config.h"

void Config::readConfig() {
	moduleEnabled = GetPrivateProfileIntA("general", "EnableModule", 1, iniFile.c_str());
	devConsoleEnabled = GetPrivateProfileIntA("general", "EnableDevConsole", 1, iniFile.c_str());

	char section[16];
	char teamname[32];
	char filename[MAX_PATH];
	for(int i=0; i < 128; i++) {
		sprintf_s(section, "team%d", i);
		GetPrivateProfileStringA(section, "TeamName", "", teamname, sizeof(teamname), iniFile.c_str());
		GetPrivateProfileStringA(section, "FileName", "", filename, sizeof(filename), iniFile.c_str());
		int animationDelay = GetPrivateProfileIntA(section, "AnimationDelay", 0,  iniFile.c_str());
		if(animationDelay < 0) animationDelay = 0;
		if(strlen(teamname) && strlen(filename)) {
			teamSettingsByName[teamname] = std::make_shared<TeamBarSettings>(filename, animationDelay);
		}
	}

	for(auto & alliance : alliances) {
		GetPrivateProfileStringA(alliance.c_str(), "FileName", "", filename, sizeof(filename), iniFile.c_str());
		int animationDelay = GetPrivateProfileIntA(alliance.c_str(), "AnimationDelay", 0,  iniFile.c_str());
		if(animationDelay < 0) animationDelay = 0;
		if(strlen(filename)) {
			teamSettingsByName[alliance + defaultSuffix] = std::make_shared<TeamBarSettings>(filename, animationDelay);
		}
	}
}

bool Config::isDevConsoleEnabled() {
	return devConsoleEnabled;
}

bool Config::isModuleEnabled() {
	return moduleEnabled;
}

std::map<std::string, std::shared_ptr<Config::TeamBarSettings>> &Config::getTeamSettingsByName() {
	return teamSettingsByName;
}

std::map<int, std::shared_ptr<Config::TeamBarSettings>> &Config::getTeamSettingsById() {
	return teamSettingsById;
}
