#include "W2App.h"
#include "Hooks.h"
#include "BitmapImage.h"
#include "Config.h"

DWORD origInitializeW2App;
DWORD __stdcall W2App::hookInitializeW2App(DWORD DD_Game_a2, DWORD DD_Display_a3, DWORD DS_Sound_a4, DWORD DD_Keyboard_a5, DWORD DD_Mouse_a6, DWORD WAV_CDrom_a7, DWORD WS_GameNet_a8) {
	DWORD DD_W2Wrapper, retv;
	_asm mov DD_W2Wrapper, edi

	addrDDGame = DD_Game_a2;
	addrDDDisplay = DD_Display_a3;
	addrDSSound = DS_Sound_a4;
	addrDDKeyboard = DD_Keyboard_a5;
	addrDDMouse = DD_Mouse_a6;
	addrWavCDRom = WAV_CDrom_a7;
	addrWSGameNet = WS_GameNet_a8;

	_asm push WS_GameNet_a8
	_asm push WAV_CDrom_a7
	_asm push DD_Mouse_a6
	_asm push DD_Keyboard_a5
	_asm push DS_Sound_a4
	_asm push DD_Display_a3
	_asm push DD_Game_a2
	_asm mov edi, DD_W2Wrapper
	_asm call origInitializeW2App
	_asm mov retv, eax

	return retv;
}

//https://stackoverflow.com/a/14679003
std::string ReplaceString(std::string subject, const std::string& search, const std::string& replace) {
	size_t pos = 0;
	while((pos = subject.find(search, pos)) != std::string::npos) {
		subject.replace(pos, search.length(), replace);
		pos += replace.length();
	}
	return subject;
}

DWORD (__stdcall *origConstructGameGlobal)(int ddgame);
DWORD __stdcall W2App::hookConstructGameGlobal(DWORD ddgame) {
	auto ret = origConstructGameGlobal(ddgame);
	auto addrGameGlobal = *(DWORD*)(ddgame + 0x488);
	auto & settingsById = Config::getTeamSettingsById();
	auto & settingsByName = Config::getTeamSettingsByName();

	for(int i=1; i < 7; i++) {
		auto teamName = (char*)((addrGameGlobal + 0x4114) + 0x51C * i);
		auto addrDDMain = *(DWORD*)(addrGameGlobal + 0x24);
		auto allianceId = *(WORD*)(addrDDMain + 0xBB8 * i - 0x767);
		if(allianceId < 0 || allianceId > Config::alliances.size()) {
			printf("Invalid alliance id: %d\n", allianceId);
			continue;
		}
		auto allianceName = Config::alliances[allianceId];
		printf("addr: 0x%08X slot: %d name: %s alliance: %d (%s)\n", (DWORD)teamName, i, teamName, allianceId, allianceName.c_str());

		auto it = settingsByName.find(teamName); //find configured team
		if(it == settingsByName.end())
			it = settingsByName.find(allianceName + Config::defaultSuffix); //fallback to alliance color

		// only load images for teams present in game
		if(it != settingsByName.end()) {
			auto settings = it->second;
			std::string filename = ReplaceString(settings->filename, "%COLOR%", allianceName);
			auto img = BitmapImage::callLoadImgFromFile(0, 0, filename.c_str());
			if(!img) {
				printf("Failed to load team health bar image: %s\n", filename.c_str());
				continue;
			}
			settings->bitmapImage = img;
			if(img->max_width_dword14 == 256 && img->max_height_dword18 == 8) {
				settings->animationType = Config::TeamBarSettings::Flame;
			} else if (img->max_width_dword14 >= 128 && img->max_height_dword18 >= TeamBar::barHeight && img->max_height_dword18 % TeamBar::barHeight == 0) {
				settings->animationType = Config::TeamBarSettings::Custom;
				settings->animationFrames = img->max_height_dword18 / TeamBar::barHeight;
			} else {
				printf("Failed to determine health bar type: %s - width: %d height: %d\n", filename.c_str(), img->max_width_dword14, img->max_height_dword18);
				continue;
			}
			settingsById[i] = settings;
		}
	}

	return ret;
}

DWORD (__fastcall *origDestroyGameGlobal)(int This, int EDX);
DWORD __fastcall W2App::hookDestroyGameGlobal(int This, int EDX) {
	auto ret = origDestroyGameGlobal(This, EDX);
	addrDDDisplay = addrDSSound = addrDDKeyboard = addrDDMouse = addrWavCDRom = addrWSGameNet = addrDDGame = 0;
	auto & settingsById = Config::getTeamSettingsById();
	for(auto & it : settingsById) {
		waFree(it.second->bitmapImage);
		it.second->bitmapImage = nullptr;
		it.second->currentFrame = 0;
		it.second->counter = 0;
	}
	settingsById.clear();
	return ret;
}

void (__cdecl *origWaFree)(void * ptr);
void W2App::waFree(void *ptr) {
	origWaFree(ptr);
}

void W2App::install() {
	auto & hooks = Hooks::getInstance();

	DWORD addrInitializeW2App = hooks.findSignature("InitializeW2App", "\x6A\xFF\x64\xA1\x00\x00\x00\x00\x68\x00\x00\x00\x00\x50\x64\x89\x25\x00\x00\x00\x00\x53\x8B\x5C\x24\x1C\x55\x8B\x6C\x24\x1C\x56\x8B\x74\x24\x1C\x33\xC0\x89\x86\x00\x00\x00\x00\x89\x44\x24\x14\x89\x86\x00\x00\x00\x00\x8B\xC7\xC7\x06\x00\x00\x00\x00\x89\x9E\x00\x00\x00\x00\x89\xAE\x00\x00\x00\x00\xE8\x00\x00\x00\x00", "????????x????xxxx????xxxxxxxxxxxxxxxxxxx????xxxxxx????xxxx????xx????xx????x????"); //0x56D840
	DWORD addrConstructGameGlobal = hooks.findSignature("ConstructGameGlobal", "\x6A\xFF\x68\x00\x00\x00\x00\x64\xA1\x00\x00\x00\x00\x50\x64\x89\x25\x00\x00\x00\x00\x83\xEC\x24\x53\x55\x8B\x6C\x24\x3C\x8B\x85\x00\x00\x00\x00\x8B\x48\x24", "???????xx????xxxx????xxxxxxxxxxx????xxx"); //0x5263C0);
	DWORD addrDestroyGameGlobal = hooks.findSignature("DestroyGameGlobal", "\x6A\xFF\x68\x00\x00\x00\x00\x64\xA1\x00\x00\x00\x00\x50\x64\x89\x25\x00\x00\x00\x00\x51\x56\x8B\xF1\x89\x74\x24\x04\xC7\x06\x00\x00\x00\x00\x8B\xC6\xC7\x44\x24\x00\x00\x00\x00\x00\xE8\x00\x00\x00\x00\xA1\x00\x00\x00\x00\x83\x78\x5C\x00\x75\x07\x8B\xC6\xE8\x00\x00\x00\x00\x8B\xC6\xE8\x00\x00\x00\x00\x8B\x86\x00\x00\x00\x00\x85\xC0\x74\x09\x50\xE8\x00\x00\x00\x00\x83\xC4\x04", "???????xx????xxxx????xxxxxxxxxx????xxxxx?????x????x????xxxxxxxxx????xxx????xx????xxxxxx????xxx");//, 0x56D900);
	origWaFree = (void (__cdecl *)(void *))hooks.findSignature("WaFree", "\x6A\x0C\x68\x00\x00\x00\x00\xE8\x00\x00\x00\x00\x8B\x75\x08\x85\xF6\x74\x75\x83\x3D\x00\x00\x00\x00\x00\x75\x43\x6A\x04\xE8\x00\x00\x00\x00\x59\x83\x65\xFC\x00", "???????x????xxxxxxxxx?????xxxxx????xxxxx");//, 0x005D05D2);


	hooks.hook("InitializeW2App", addrInitializeW2App, (DWORD*)&hookInitializeW2App, (DWORD*)&origInitializeW2App);
	hooks.hook("ConstructGameGlobal", addrConstructGameGlobal, (DWORD*)&hookConstructGameGlobal, (DWORD*)&origConstructGameGlobal);
	hooks.hook("DestroyGameGlobal", addrDestroyGameGlobal, (DWORD*)&hookDestroyGameGlobal, (DWORD*)&origDestroyGameGlobal);
}

DWORD W2App::getAddrDdGame() {
	return addrDDGame;
}

DWORD W2App::getAddrDdDisplay() {
	return addrDDDisplay;
}

DWORD W2App::getAddrDsSound() {
	return addrDSSound;
}

DWORD W2App::getAddrDdKeyboard() {
	return addrDDKeyboard;
}

DWORD W2App::getAddrDdMouse() {
	return addrDDMouse;
}

DWORD W2App::getAddrWavCdRom() {
	return addrWavCDRom;
}

DWORD W2App::getAddrWsGameNet() {
	return addrWSGameNet;
}
